@@ -11,6 +11,8 @@ SQL update of points that is way faster than player_points action in Admin.
 DRAWBACK: only tested on MySQL, may be MySQL specfic update.
 SOLUTION: Find DBIC way of doing it?  Use sub-query.
 
+Note: sqlite3 does not like the syntax on this update
+
 =cut
 
 sub update_points {
@@ -20,21 +22,33 @@ sub update_points {
         sub {
             my $self = shift;
             my $dbh  = shift;
-            my $sth  = $dbh->prepare('
-                update player player, 
+            my $sth  = $dbh->prepare('delete from region_score;');
+            $sth->execute;
+            $sth = $dbh->prepare('
+                insert into region_score
+                (player, region)
+                select player.id, region.id
+                from player, region
+                where player.active = 1;'
+            );
+            $sth->execute;
+            $sth = $dbh->prepare('
+                update region_score region_score,
                 (
                 select  player_picks.player,
-                        sum(game.round*(5 + game.lower_seed*team.seed)) as points
-                  from picks player_picks, picks perfect_picks, game game, team team
+                        sum(game.round*(5 + game.lower_seed*team.seed)) as points,
+                        team.region as region
+                  from pick player_picks, pick perfect_picks, game game, team team
                  where perfect_picks.pick   = player_picks.pick 
                    and perfect_picks.game   = player_picks.game 
                    and player_picks.game    = game.id
                    and player_picks.pick    = team.id
                    and perfect_picks.player = 1
-                   group by player_picks.player
+                   group by player_picks.player, team.region
                 )  computed_player_points
-                set player.points = computed_player_points.points
-                where player.id = computed_player_points.player
+                set region_score.points = computed_player_points.points
+                where region_score.player = computed_player_points.player
+                  and region_score.region = computed_player_points.region
                 ;'
             );
             $sth->execute();
@@ -57,11 +71,11 @@ sub count_region_picks {
             my $self = shift;
             my $dbh  = shift;
             my $sth  = $dbh->prepare('
-            select region.id, count(*) from picks  
-            join team on picks.pick = team.id  
-            join region on team.region = region.id  
-            join game on picks.game = game.id  
-            where game.round < 5 and player = ? 
+            select region.id, count(*) from pick
+            join team on pick.pick = team.id
+            join region on team.region = region.id
+            join game on pick.game = game.id
+            where game.round < 5 and player = ?
             group by region.id
             ;'
             );
@@ -92,7 +106,7 @@ sub count_player_picks {
             my $dbh  = shift;
             my $sth  = $dbh->prepare('
             select player.id, count(*) from player
-            join picks on player.id = picks.player
+            join pick on player.id = pick.player
             group by player.id
             ;'
             );
@@ -121,9 +135,9 @@ sub count_player_picks_correct {
         sub {
             my $self = shift;
             my $dbh  = shift;
-            my $sth  = $dbh->prepare('                   
+            my $sth  = $dbh->prepare('
             select player_picks.player, count(*)
-              from picks player_picks, picks perfect_picks, game game, team team
+              from pick player_picks, pick perfect_picks, game game, team team
              where perfect_picks.pick   = player_picks.pick 
                and perfect_picks.game   = player_picks.game 
                and player_picks.game    = game.id
@@ -143,6 +157,42 @@ sub count_player_picks_correct {
     );
 }
 
+=heads2 count_player_picks_upset
+
+Count up how many upset picks a player has made correct so far.
+
+=cut
+
+sub count_player_picks_upset {
+    my $self = shift;
+    my $storage = $self->schema->storage;
+    return $storage->dbh_do(
+        sub {
+            my $self = shift;
+            my $dbh  = shift;
+            my $sth  = $dbh->prepare('
+            select player_picks.player, count(*)
+              from pick player_picks, pick perfect_picks, game game, team team
+             where perfect_picks.pick   = player_picks.pick
+               and perfect_picks.game   = player_picks.game
+               and player_picks.game    = game.id
+               and player_picks.pick    = team.id
+               and perfect_picks.player = 1
+               and game.lower_seed      = 1
+          group by player_picks.player
+          ;'
+          );
+          $sth->execute() or die $sth->errstr;
+          my $upset_picks_per_player = {};
+          my $result = $sth->fetchall_arrayref;
+          foreach my $row (@{$result}) {
+              $upset_picks_per_player->{$row->[0]} = $row->[1];
+          }
+          return $upset_picks_per_player;
+        }
+    );
+}
+
 =heads2 count_final4_picks
 
 Count up how many picks a player has made in the final 4 (3 total).
@@ -159,8 +209,8 @@ sub count_final4_picks {
             my $dbh  = shift;
             my $sth  = $dbh->prepare('
             select count(*) from player
-            join picks on player.id = picks.player
-            where picks.game > 60
+            join pick on player.id = pick.player
+            where pick.game > 60
             and player.id = ?
             ;'
             );
